unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ToolWin, ComCtrls, Menus, StdCtrls, Grids, ExtCtrls, ImgList;

type
  TMainForm = class(TForm)
    StatusBar: TStatusBar;
    ToolBar: TToolBar;
    TB_New: TToolButton;
    TB_Open: TToolButton;
    TB_SaveAs: TToolButton;
    TB_sep1: TToolButton;
    TB_Run: TToolButton;
    TB_TraceStatus: TToolButton;
    TB_sep2: TToolButton;
    MainMenu1: TMainMenu;
    MainMenu_File: TMenuItem;
    MM_File_New: TMenuItem;
    MM_File_Open: TMenuItem;
    MM_File_Save: TMenuItem;
    MainMenu_Run: TMenuItem;
    MM_File_Exit: TMenuItem;
    TB_Exit: TToolButton;
    ImageList1: TImageList;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    MM_File_SaveAs: TMenuItem;
    TB_Save: TToolButton;
    MainPanel: TPanel;
    ProgramBox: TGroupBox;
    ProgramMemo: TMemo;
    DataBox: TGroupBox;
    DataMemo: TMemo;
    TraceBox: TGroupBox;
    ParserBox: TGroupBox;
    ParserGrid: TStringGrid;
    TraceMemo: TMemo;

    procedure MM_File_NewClick(Sender: TObject);
    procedure MM_File_OpenClick(Sender: TObject);
    procedure MM_File_SaveClick(Sender: TObject);
    procedure MM_File_SaveAsClick(Sender: TObject);
    procedure MM_File_ExitClick(Sender: TObject);

    procedure MainMenu_RunClick(Sender: TObject);

    procedure TB_TraceStatusClick(Sender: TObject);

    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);

    procedure ShowHint(Sender: TObject);
    procedure FormResize(Sender: TObject);

  private
    { Private declarations }

    isRunning,
    isRunningBreaked : boolean;
    Trace_Program    : boolean;
    FileName         : string;

    procedure NewFile;
    procedure OpenFile;
    procedure SaveFile;
    procedure SaveFileAs (f : boolean);

    procedure ParseProgram;
    procedure RunProgram;

  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

//-----------------------------------------------------------------------

procedure TMainForm.NewFile;
begin
  //   :

  FileName := ' .prolanm';
  Caption  := 'PROLAN/M : ' + FileName;

  //  :
  //  (,)
  with ProgramMemo do
  begin
    Clear;
    Lines.Add (':');
    Lines.Add ('(,)');
    Modified := false;
  end;

  //   
  with DataMemo do
  begin
    Clear;
    Lines.Add (' ');
  end;

  //    
  with TraceMemo do
  begin
    Clear;
    Lines.Add ('  ');
  end;
end;

procedure TMainForm.OpenFile;
  var
    F : TextFile;
    s : string;
    pdt : (programme, data_string, traces);
    pm1, dm1, tm1 : boolean;
begin
  Caption  := 'PROLAN/M : ' + FileName;

  AssignFile (F, FileName);
  Reset (F);

  pdt := programme;

  pm1 := true;
  dm1 := true;
  tm1 := true;

  ProgramMemo.Lines.Clear; ProgramMemo.Lines.BeginUpdate;
  DataMemo   .Lines.Clear; DataMemo   .Lines.BeginUpdate;
  TraceMemo  .Lines.Clear; TraceMemo  .Lines.BeginUpdate;

  ReadLn (F, s);
  while not Eof(F) do
  begin
    if      s = '[]'                    then pdt := programme
    else if s = '[ ]'                then pdt := data_string
    else if s = '[  ]' then pdt := traces
    else case pdt of
           programme   : if pm1 then
                         begin
                           pm1 := false;
                           ProgramMemo.Lines.Text := ProgramMemo.Lines.Text + s;
                         end
                         else ProgramMemo.Lines.Text := ProgramMemo.Lines.Text + #13#10 + s;

           data_string : if dm1 then
                         begin
                           dm1 := false;
                           DataMemo.Lines.Text := DataMemo.Lines.Text + s;
                         end
                         else DataMemo.Lines.Text := DataMemo.Lines.Text + #13#10 + s;

           traces      : if tm1 then
                         begin
                           tm1 := false;
                           TraceMemo.Lines.Text := TraceMemo.Lines.Text + s;
                         end
                         else TraceMemo.Lines.Text := TraceMemo.Lines.Text + #13#10 + s;
         end;

    ReadLn (F, s);
  end;
  CloseFile (F);

  ProgramMemo.Lines.EndUpdate;
  DataMemo   .Lines.EndUpdate;
  TraceMemo  .Lines.EndUpdate;
end;

procedure TMainForm.SaveFile;
  var
    F : TextFile;
begin
//   :  +   + 

  Caption  := 'PROLAN/M : ' + FileName;

  AssignFile (F, FileName);
  Rewrite (F);

  WriteLn (F, '[]'#13#10,
              ProgramMemo.Lines.Text,
              #13#10'[ ]'#13#10,
              DataMemo.Lines.Text,
              #13#10'[  ]'#13#10,
              TraceMemo.Lines.Text);

  CloseFile (F);

  ProgramMemo.Modified := false;
end;

procedure TMainForm.SaveFileAs (f : boolean);
  var
    Ok : boolean;
begin
  isRunningBreaked := true;

  if not f then Ok := true
  else
  begin
{
    Ok := MessageDlg (' .'#13#10#13#10 +
                      '?', mtConfirmation,
                      [mbYes, mbNo], 0) = mrYes;
}
    Ok := Application.MessageBox (' .'^M^M +
                                  '?',
                                  '',
                                  MB_YesNo) = IDYes;
  end;

  if Ok then
  begin
    SaveDialog.FileName := FileName;
    if SaveDialog.Execute then
    begin
      FileName := SaveDialog.FileName;
      SaveFile;
    end;
  end;
end;

//-----------------------------------------------------------------------

procedure TMainForm.FormResize(Sender: TObject);
  var
    W2, W1, H1 : integer;
begin
  MainPanel.Hide;

  if Width  < 400 then Width  := 400;
  if Height < 300 then Height := 300;

  W1 := (Width - 8) div 3;
  W2 := W1 * 2- 5;
  H1 := (MainPanel.Height - 8 - 9) div 5;

  ProgramBox.Top    :=  4;
  ProgramBox.Left   :=  4;
  ProgramBox.Height := H1 * 2;
  ProgramBox.Width  := W2;

  DataBox.Top    := ProgramBox.Top + ProgramBox.Height + 5;
  DataBox.Left   :=  4;
  DataBox.Height := H1;
  DataBox.Width  := W2;

  TraceBox.Top    := DataBox.Top + DataBox.Height + 5;
  TraceBox.Left   :=  4;
  TraceBox.Height := H1 * 2;
  TraceBox.Width  := W2;

  ParserBox.Top    := ProgramBox.Top;
  ParserBox.Height := TraceBox.Top + TraceBox.Height - ParserBox.Top;
  ParserBox.Left   := ProgramBox.Width + 5;
  ParserBox.Width  := W1;
  ParserGrid.ColWidths[0] := (ParserGrid.Width div 2) - 1;
  ParserGrid.ColWidths[1] := ParserGrid.ColWidths[0];

  MainPanel.Show;
end;

procedure TMainForm.MM_File_OpenClick(Sender: TObject);
begin
  isRunningBreaked  := true;

  if ProgramMemo.Modified then SaveFileAs (true);

  OpenDialog.FileName := FileName;
  if OpenDialog.Execute then
  begin
    FileName := OpenDialog.FileName;
    OpenFile;
  end;
end;

procedure TMainForm.MM_File_SaveClick(Sender: TObject);
begin
  isRunningBreaked  := true;

  SaveFile;
end;

procedure TMainForm.MM_File_SaveAsClick(Sender: TObject);
begin
  isRunningBreaked  := true;

  SaveFileAs (false);
end;

procedure TMainForm.TB_TraceStatusClick(Sender: TObject);
begin
  Trace_Program := not Trace_Program;
  if Trace_Program
  then TB_TraceStatus.Hint := ' '
  else TB_TraceStatus.Hint := ' ';
end;

procedure TMainForm.MM_File_NewClick(Sender: TObject);
begin
  isRunningBreaked  := true;

  if ProgramMemo.Modified then SaveFileAs (true);

  NewFile;
end;

procedure TMainForm.MainMenu_RunClick(Sender: TObject);
begin
  if isRunning then
  begin
    isRunningBreaked  := true;
  end
  else
  begin
    isRunning         := true;
    TB_Run.ImageIndex := 7;
    TB_Run.Hint       := ' ';
    ProgramMemo.Enabled := false;
    DataMemo.   Enabled := false;
    TraceMemo.  Enabled := false;
    ParserGrid. Enabled := false;

    ParseProgram;
    RunProgram;

    isRunning         := false;
    TB_Run.ImageIndex := 4;
    TB_Run.Hint       := ' ';
    ProgramMemo.Enabled := true;
    DataMemo.   Enabled := true;
    TraceMemo.  Enabled := true;
    ParserGrid. Enabled := true;
  end;
end;

procedure TMainForm.MM_File_ExitClick(Sender: TObject);
begin
//   
  isRunningBreaked  := true;

  Close;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  MainPanel.Visible := false;
  Application.OnHint := ShowHint;

  NewFile;
  if ParamCount > 0 then
  begin
    FileName := ParamStr (1);
    OpenFile;
  end;
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if ProgramMemo.Modified then SaveFileAs (true);
end;

procedure TMainForm.ShowHint(Sender: TObject);
begin
  StatusBar.SimpleText := Application.Hint;
end;


//=======================================================================

procedure TMainForm.ParseProgram;
  var
    c          : char;
    i, ls      : longint;
    first_line : boolean;
    T, D       : boolean;
    LL, RR     : string;
begin
//   

  isRunningBreaked  := true;

  ls := Length (ProgramMemo.Lines.Text);
  first_line := true;

  T  := false; D  := false;
  LL := '';    RR := '';

  with ParserGrid do
  begin
    Hide;
//  Rows [0].BeginUpdate;
//  Rows [1].BeginUpdate;
    RowCount := 1;
    Cells [0, 0] := '';
    Cells [1, 0] := '';
    ColWidths[0] := (Width div 2) - 1;
    ColWidths[1] := ColWidths[0];
  end;

  for i := 1 to ls - 2 do
  begin
    c := ProgramMemo.Lines.Text [i];
    case c of
      #10, #13 : ;            //   - 

      '(' : if (ProgramMemo.Lines.Text [i + 1] = ',') and
               (ProgramMemo.Lines.Text [i + 2] = ')')
            then  break       // (,) -  
            else  T := true;  // (   -   

      ',' : begin             //  ,  -   
              T := false; D := true;

              if first_line then first_line := false
              else ParserGrid.RowCount := ParserGrid.RowCount + 1;

              ParserGrid.Cells [0, ParserGrid.RowCount - 1] := LL;
              LL := '';
            end;

      ')' : begin             //   ) -   
              if T then       // (?) -   
              begin
                T := false;

                if first_line then first_line := false
                else ParserGrid.RowCount := ParserGrid.RowCount + 1;

                ParserGrid.Cells [0, ParserGrid.RowCount - 1] := LL;
                LL := '';
              end;

              if not D
              then ParserGrid.Cells [1, ParserGrid.RowCount - 1] := ''
              else
              begin
                D := false;
                ParserGrid.Cells [1, ParserGrid.RowCount - 1] := RR;
                RR := '';
              end;
            end;
    else
      if      T then LL := LL + c
      else if D then RR := RR + c;
    end;
  end;

  with ParserGrid do
  begin
//  Rows [0].EndUpdate;
//  Rows [1].EndUpdate;
    Show;
  end;
end;

procedure TMainForm.RunProgram;
  label Further, Exit;

  var
    isDataNotChanged : boolean;
    i                : integer;
    DataStr, LL      : string;
    LLpos            : integer;
begin
{   :

   
   | __ = ;
   |    _
   | 
   | |  i  1  ((_)-(.))
   | | 
   | | |  _ [i..i+(.)] = .
   | | | |  _ := _ [1..i]
   | | | |                   + .
   | | | |                   + _ [i..(_)];
   | | | |    __ = ;
   | | | |      
   | | | 
   | | 
   | 
   | <<>> : ;
     (__ = );
}

  TraceMemo.Clear;
  DataStr := DataMemo.Text;

  isRunningBreaked := false;

  repeat
    Application.ProcessMessages;

    isDataNotChanged := true;
    if Trace_Program then TraceMemo.Lines.Add (DataStr);

    for i := 0 to ParserGrid.RowCount - 1 do
    begin
      if isRunningBreaked then goto Exit;

      LL := ParserGrid.Cells [0, i];
      LLpos := Pos (LL, DataStr);
      if LLpos > 0 then
      begin
        isDataNotChanged := false;
        Delete (DataStr, LLpos, Length (LL));
        Insert (ParserGrid.Cells [1, i], DataStr, LLpos);
        goto Further;
      end;
    end;
  Further:;
  until isDataNotChanged;

Exit:
  if not Trace_Program then TraceMemo.Lines.Add (DataStr);

end;

end.

